@@ -1,3 +1,4 @@
+//go:build !windows && !darwin
 // +build !windows,!darwin
 
 /*
@@ -22,11 +23,10 @@ import (
 	"archive/tar"
 	"bytes"
 	"context"
+	"errors"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"os"
-	"os/exec"
 	"path/filepath"
 	"testing"
 	"time"
@@ -37,7 +37,7 @@ import (
 	"github.com/containerd/containerd/pkg/testutil"
 	"github.com/containerd/continuity/fs"
 	"github.com/containerd/continuity/fs/fstest"
-	"github.com/pkg/errors"
+	exec "golang.org/x/sys/execabs"
 )
 
 const tarCmd = "tar"
@@ -233,17 +233,17 @@ func TestBreakouts(t *testing.T) {
 	tc := tartest.TarContext{}.WithUIDGID(os.Getuid(), os.Getgid()).WithModTime(time.Now().UTC())
 	expected := "unbroken"
 	unbrokenCheck := func(root string) error {
-		b, err := ioutil.ReadFile(filepath.Join(root, "etc", "unbroken"))
+		b, err := os.ReadFile(filepath.Join(root, "etc", "unbroken"))
 		if err != nil {
-			return errors.Wrap(err, "failed to read unbroken")
+			return fmt.Errorf("failed to read unbroken: %w", err)
 		}
 		if string(b) != expected {
-			return errors.Errorf("/etc/unbroken: unexpected value %s, expected %s", b, expected)
+			return fmt.Errorf("/etc/unbroken: unexpected value %s, expected %s", b, expected)
 		}
 		return nil
 	}
 	errFileDiff := errors.New("files differ")
-	td, err := ioutil.TempDir("", "test-breakouts-")
+	td, err := os.MkdirTemp("", "test-breakouts-")
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -257,7 +257,7 @@ func TestBreakouts(t *testing.T) {
 			}
 
 			if got := fi.Mode() & os.ModeSymlink; got != os.ModeSymlink {
-				return errors.Errorf("%s should be symlink", fi.Name())
+				return fmt.Errorf("%s should be symlink", fi.Name())
 			}
 			return nil
 		}
@@ -285,7 +285,7 @@ func TestBreakouts(t *testing.T) {
 			}
 
 			if t1 != t2 {
-				return errors.Wrapf(errFileDiff, "%#v and %#v", t1, t2)
+				return fmt.Errorf("%#v and %#v: %w", t1, t2, errFileDiff)
 			}
 			return nil
 		}
@@ -310,7 +310,7 @@ func TestBreakouts(t *testing.T) {
 				return err
 			}
 			if !os.SameFile(s1, s2) {
-				return errors.Wrapf(errFileDiff, "%#v and %#v", s1, s2)
+				return fmt.Errorf("%#v and %#v: %w", s1, s2, errFileDiff)
 			}
 			return nil
 		}
@@ -330,12 +330,12 @@ func TestBreakouts(t *testing.T) {
 	}
 	fileValue := func(f1 string, content []byte) func(string) error {
 		return func(root string) error {
-			b, err := ioutil.ReadFile(filepath.Join(root, f1))
+			b, err := os.ReadFile(filepath.Join(root, f1))
 			if err != nil {
 				return err
 			}
 			if !bytes.Equal(b, content) {
-				return errors.Errorf("content differs: expected %v, got %v", content, b)
+				return fmt.Errorf("content differs: expected %v, got %v", content, b)
 			}
 			return nil
 		}
@@ -420,12 +420,12 @@ func TestBreakouts(t *testing.T) {
 				tc.File("/localetc/emptied", []byte{}, 0644),
 			),
 			validator: func(root string) error {
-				b, err := ioutil.ReadFile(filepath.Join(root, "etc", "emptied"))
+				b, err := os.ReadFile(filepath.Join(root, "etc", "emptied"))
 				if err != nil {
-					return errors.Wrap(err, "failed to read unbroken")
+					return fmt.Errorf("failed to read unbroken: %w", err)
 				}
 				if len(b) > 0 {
-					return errors.Errorf("/etc/emptied: non-empty")
+					return errors.New("/etc/emptied: non-empty")
 				}
 				return nil
 			},
@@ -754,11 +754,11 @@ func TestBreakouts(t *testing.T) {
 			name: "HardlinkSymlinkChmod",
 			w: func() tartest.WriterToTar {
 				p := filepath.Join(td, "perm400")
-				if err := ioutil.WriteFile(p, []byte("..."), 0400); err != nil {
+				if err := os.WriteFile(p, []byte("..."), 0400); err != nil {
 					t.Fatal(err)
 				}
 				ep := filepath.Join(td, "also-exists-outside-root")
-				if err := ioutil.WriteFile(ep, []byte("..."), 0640); err != nil {
+				if err := os.WriteFile(ep, []byte("..."), 0640); err != nil {
 					t.Fatal(err)
 				}
 
@@ -774,7 +774,7 @@ func TestBreakouts(t *testing.T) {
 					return err
 				}
 				if perm := fi.Mode() & os.ModePerm; perm != 0400 {
-					return errors.Errorf("%s perm changed from 0400 to %04o", p, perm)
+					return fmt.Errorf("%s perm changed from 0400 to %04o", p, perm)
 				}
 				return nil
 			},
@@ -800,7 +800,7 @@ func TestApplyTar(t *testing.T) {
 					return err
 				}
 				if _, err := os.Stat(p); err != nil {
-					return errors.Wrapf(err, "failure checking existence for %v", d)
+					return fmt.Errorf("failure checking existence for %v: %w", d, err)
 				}
 			}
 			return nil
@@ -842,25 +842,25 @@ func TestApplyTar(t *testing.T) {
 }
 
 func testApply(a fstest.Applier) error {
-	td, err := ioutil.TempDir("", "test-apply-")
+	td, err := os.MkdirTemp("", "test-apply-")
 	if err != nil {
-		return errors.Wrap(err, "failed to create temp dir")
+		return fmt.Errorf("failed to create temp dir: %w", err)
 	}
 	defer os.RemoveAll(td)
-	dest, err := ioutil.TempDir("", "test-apply-dest-")
+	dest, err := os.MkdirTemp("", "test-apply-dest-")
 	if err != nil {
-		return errors.Wrap(err, "failed to create temp dir")
+		return fmt.Errorf("failed to create temp dir: %w", err)
 	}
 	defer os.RemoveAll(dest)
 
 	if err := a.Apply(td); err != nil {
-		return errors.Wrap(err, "failed to apply filesystem changes")
+		return fmt.Errorf("failed to apply filesystem changes: %w", err)
 	}
 
 	tarArgs := []string{"c", "-C", td}
 	names, err := readDirNames(td)
 	if err != nil {
-		return errors.Wrap(err, "failed to read directory names")
+		return fmt.Errorf("failed to read directory names: %w", err)
 	}
 	tarArgs = append(tarArgs, names...)
 
@@ -868,34 +868,34 @@ func testApply(a fstest.Applier) error {
 
 	arch, err := cmd.StdoutPipe()
 	if err != nil {
-		return errors.Wrap(err, "failed to create stdout pipe")
+		return fmt.Errorf("failed to create stdout pipe: %w", err)
 	}
 
 	if err := cmd.Start(); err != nil {
-		return errors.Wrap(err, "failed to start command")
+		return fmt.Errorf("failed to start command: %w", err)
 	}
 
 	if _, err := Apply(context.Background(), dest, arch); err != nil {
-		return errors.Wrap(err, "failed to apply tar stream")
+		return fmt.Errorf("failed to apply tar stream: %w", err)
 	}
 
 	return fstest.CheckDirectoryEqual(td, dest)
 }
 
 func testBaseDiff(a fstest.Applier) error {
-	td, err := ioutil.TempDir("", "test-base-diff-")
+	td, err := os.MkdirTemp("", "test-base-diff-")
 	if err != nil {
-		return errors.Wrap(err, "failed to create temp dir")
+		return fmt.Errorf("failed to create temp dir: %w", err)
 	}
 	defer os.RemoveAll(td)
-	dest, err := ioutil.TempDir("", "test-base-diff-dest-")
+	dest, err := os.MkdirTemp("", "test-base-diff-dest-")
 	if err != nil {
-		return errors.Wrap(err, "failed to create temp dir")
+		return fmt.Errorf("failed to create temp dir: %w", err)
 	}
 	defer os.RemoveAll(dest)
 
 	if err := a.Apply(td); err != nil {
-		return errors.Wrap(err, "failed to apply filesystem changes")
+		return fmt.Errorf("failed to apply filesystem changes: %w", err)
 	}
 
 	arch := Diff(context.Background(), "", td)
@@ -903,27 +903,27 @@ func testBaseDiff(a fstest.Applier) error {
 	cmd := exec.Command(tarCmd, "x", "-C", dest)
 	cmd.Stdin = arch
 	if err := cmd.Run(); err != nil {
-		return errors.Wrap(err, "tar command failed")
+		return fmt.Errorf("tar command failed: %w", err)
 	}
 
 	return fstest.CheckDirectoryEqual(td, dest)
 }
 
 func testDiffApply(appliers ...fstest.Applier) error {
-	td, err := ioutil.TempDir("", "test-diff-apply-")
+	td, err := os.MkdirTemp("", "test-diff-apply-")
 	if err != nil {
-		return errors.Wrap(err, "failed to create temp dir")
+		return fmt.Errorf("failed to create temp dir: %w", err)
 	}
 	defer os.RemoveAll(td)
-	dest, err := ioutil.TempDir("", "test-diff-apply-dest-")
+	dest, err := os.MkdirTemp("", "test-diff-apply-dest-")
 	if err != nil {
-		return errors.Wrap(err, "failed to create temp dir")
+		return fmt.Errorf("failed to create temp dir: %w", err)
 	}
 	defer os.RemoveAll(dest)
 
 	for _, a := range appliers {
 		if err := a.Apply(td); err != nil {
-			return errors.Wrap(err, "failed to apply filesystem changes")
+			return fmt.Errorf("failed to apply filesystem changes: %w", err)
 		}
 	}
 
@@ -931,18 +931,18 @@ func testDiffApply(appliers ...fstest.Applier) error {
 	if len(appliers) > 1 {
 		for _, a := range appliers[:len(appliers)-1] {
 			if err := a.Apply(dest); err != nil {
-				return errors.Wrap(err, "failed to apply base filesystem changes")
+				return fmt.Errorf("failed to apply base filesystem changes: %w", err)
 			}
 		}
 	}
 
-	diffBytes, err := ioutil.ReadAll(Diff(context.Background(), dest, td))
+	diffBytes, err := io.ReadAll(Diff(context.Background(), dest, td))
 	if err != nil {
-		return errors.Wrap(err, "failed to create diff")
+		return fmt.Errorf("failed to create diff: %w", err)
 	}
 
 	if _, err := Apply(context.Background(), dest, bytes.NewReader(diffBytes)); err != nil {
-		return errors.Wrap(err, "failed to apply tar stream")
+		return fmt.Errorf("failed to apply tar stream: %w", err)
 	}
 
 	return fstest.CheckDirectoryEqual(td, dest)
@@ -950,7 +950,7 @@ func testDiffApply(appliers ...fstest.Applier) error {
 
 func makeWriterToTarTest(wt tartest.WriterToTar, a fstest.Applier, validate func(string) error, applyErr error) func(*testing.T) {
 	return func(t *testing.T) {
-		td, err := ioutil.TempDir("", "test-writer-to-tar-")
+		td, err := os.MkdirTemp("", "test-writer-to-tar-")
 		if err != nil {
 			t.Fatalf("Failed to create temp dir: %v", err)
 		}
@@ -1194,10 +1194,10 @@ func dirEntry(name string, mode int) tarEntryValidator {
 			return errors.New("not directory type")
 		}
 		if hdr.Name != name {
-			return errors.Errorf("wrong name %q, expected %q", hdr.Name, name)
+			return fmt.Errorf("wrong name %q, expected %q", hdr.Name, name)
 		}
 		if hdr.Mode != int64(mode) {
-			return errors.Errorf("wrong mode %o, expected %o", hdr.Mode, mode)
+			return fmt.Errorf("wrong mode %o, expected %o", hdr.Mode, mode)
 		}
 		return nil
 	}
@@ -1209,13 +1209,13 @@ func fileEntry(name string, expected []byte, mode int) tarEntryValidator {
 			return errors.New("not file type")
 		}
 		if hdr.Name != name {
-			return errors.Errorf("wrong name %q, expected %q", hdr.Name, name)
+			return fmt.Errorf("wrong name %q, expected %q", hdr.Name, name)
 		}
 		if hdr.Mode != int64(mode) {
-			return errors.Errorf("wrong mode %o, expected %o", hdr.Mode, mode)
+			return fmt.Errorf("wrong mode %o, expected %o", hdr.Mode, mode)
 		}
 		if !bytes.Equal(b, expected) {
-			return errors.Errorf("different file content")
+			return errors.New("different file content")
 		}
 		return nil
 	}
@@ -1227,10 +1227,10 @@ func linkEntry(name, link string) tarEntryValidator {
 			return errors.New("not link type")
 		}
 		if hdr.Name != name {
-			return errors.Errorf("wrong name %q, expected %q", hdr.Name, name)
+			return fmt.Errorf("wrong name %q, expected %q", hdr.Name, name)
 		}
 		if hdr.Linkname != link {
-			return errors.Errorf("wrong link %q, expected %q", hdr.Linkname, link)
+			return fmt.Errorf("wrong link %q, expected %q", hdr.Linkname, link)
 		}
 		return nil
 	}
@@ -1243,10 +1243,10 @@ func whiteoutEntry(name string) tarEntryValidator {
 
 	return func(hdr *tar.Header, b []byte) error {
 		if hdr.Typeflag != tar.TypeReg {
-			return errors.Errorf("not file type: %q", hdr.Typeflag)
+			return fmt.Errorf("not file type: %q", hdr.Typeflag)
 		}
 		if hdr.Name != whiteOut {
-			return errors.Errorf("wrong name %q, expected whiteout %q", hdr.Name, name)
+			return fmt.Errorf("wrong name %q, expected whiteout %q", hdr.Name, name)
 		}
 		return nil
 	}
@@ -1254,7 +1254,7 @@ func whiteoutEntry(name string) tarEntryValidator {
 
 func makeDiffTarTest(validators []tarEntryValidator, a, b fstest.Applier) func(*testing.T) {
 	return func(t *testing.T) {
-		ad, err := ioutil.TempDir("", "test-make-diff-tar-")
+		ad, err := os.MkdirTemp("", "test-make-diff-tar-")
 		if err != nil {
 			t.Fatalf("failed to create temp dir: %v", err)
 		}
@@ -1263,7 +1263,7 @@ func makeDiffTarTest(validators []tarEntryValidator, a, b fstest.Applier) func(*
 			t.Fatalf("failed to apply a: %v", err)
 		}
 
-		bd, err := ioutil.TempDir("", "test-make-diff-tar-")
+		bd, err := os.MkdirTemp("", "test-make-diff-tar-")
 		if err != nil {
 			t.Fatalf("failed to create temp dir: %v", err)
 		}
@@ -1289,7 +1289,7 @@ func makeDiffTarTest(validators []tarEntryValidator, a, b fstest.Applier) func(*
 			}
 			var b []byte
 			if hdr.Typeflag == tar.TypeReg && hdr.Size > 0 {
-				b, err = ioutil.ReadAll(tr)
+				b, err = io.ReadAll(tr)
 				if err != nil {
 					t.Fatalf("tar read file error: %v", err)
 				}
@@ -1307,9 +1307,9 @@ func makeDiffTarTest(validators []tarEntryValidator, a, b fstest.Applier) func(*
 type diffApplier struct{}
 
 func (d diffApplier) TestContext(ctx context.Context) (context.Context, func(), error) {
-	base, err := ioutil.TempDir("", "test-diff-apply-")
+	base, err := os.MkdirTemp("", "test-diff-apply-")
 	if err != nil {
-		return ctx, nil, errors.Wrap(err, "failed to create temp dir")
+		return ctx, nil, fmt.Errorf("failed to create temp dir: %w", err)
 	}
 	return context.WithValue(ctx, d, base), func() {
 		os.RemoveAll(base)
@@ -1319,32 +1319,32 @@ func (d diffApplier) TestContext(ctx context.Context) (context.Context, func(),
 func (d diffApplier) Apply(ctx context.Context, a fstest.Applier) (string, func(), error) {
 	base := ctx.Value(d).(string)
 
-	applyCopy, err := ioutil.TempDir("", "test-diffapply-apply-copy-")
+	applyCopy, err := os.MkdirTemp("", "test-diffapply-apply-copy-")
 	if err != nil {
-		return "", nil, errors.Wrap(err, "failed to create temp dir")
+		return "", nil, fmt.Errorf("failed to create temp dir: %w", err)
 	}
 	defer os.RemoveAll(applyCopy)
 	if err = fs.CopyDir(applyCopy, base); err != nil {
-		return "", nil, errors.Wrap(err, "failed to copy base")
+		return "", nil, fmt.Errorf("failed to copy base: %w", err)
 	}
 	if err := a.Apply(applyCopy); err != nil {
-		return "", nil, errors.Wrap(err, "failed to apply changes to copy of base")
+		return "", nil, fmt.Errorf("failed to apply changes to copy of base: %w", err)
 	}
 
-	diffBytes, err := ioutil.ReadAll(Diff(ctx, base, applyCopy))
+	diffBytes, err := io.ReadAll(Diff(ctx, base, applyCopy))
 	if err != nil {
-		return "", nil, errors.Wrap(err, "failed to create diff")
+		return "", nil, fmt.Errorf("failed to create diff: %w", err)
 	}
 
 	if _, err = Apply(ctx, base, bytes.NewReader(diffBytes)); err != nil {
-		return "", nil, errors.Wrap(err, "failed to apply tar stream")
+		return "", nil, fmt.Errorf("failed to apply tar stream: %w", err)
 	}
 
 	return base, nil, nil
 }
 
 func readDirNames(p string) ([]string, error) {
-	fis, err := ioutil.ReadDir(p)
+	fis, err := os.ReadDir(p)
 	if err != nil {
 		return nil, err
 	}
