@@ -23,7 +23,6 @@ import (
 	"github.com/containerd/containerd/sys/reaper"
 	runc "github.com/containerd/go-runc"
 	"github.com/containerd/ttrpc"
-	"github.com/opencontainers/runc/libcontainer/system"
 	"golang.org/x/sys/unix"
 )
 
@@ -36,7 +35,7 @@ func setupSignals() (chan os.Signal, error) {
 	// for waiting on processes
 	runc.Monitor = reaper.Default
 	// set the shim as the subreaper for all orphaned processes created by the container
-	if err := system.SetSubreaper(1); err != nil {
+	if err := reaper.SetSubreaper(1); err != nil {
 		return nil, err
 	}
 	return signals, nil
