@@ -1,3 +1,4 @@
+//go:build linux || solaris
 // +build linux solaris
 
 /*
@@ -26,7 +27,7 @@ import (
 
 func getATime(fi os.FileInfo) time.Time {
 	if st, ok := fi.Sys().(*syscall.Stat_t); ok {
-		return time.Unix(int64(st.Atim.Sec), int64(st.Atim.Nsec)) //nolint: unconvert // int64 conversions ensure the line compiles for 32-bit systems as well.
+		return time.Unix(st.Atim.Unix())
 	}
 
 	return fi.ModTime()
