@@ -1,6 +1,3 @@
-//go:build linux
-// +build linux
-
 /*
    Copyright The containerd Authors.
 
@@ -24,9 +21,7 @@ import (
 	"context"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"os"
-	"os/exec"
 	"path/filepath"
 	"runtime"
 	"strings"
@@ -38,130 +33,21 @@ import (
 	"github.com/containerd/cgroups"
 	cgroupsv2 "github.com/containerd/cgroups/v2"
 	. "github.com/containerd/containerd"
-	apievents "github.com/containerd/containerd/api/events"
 	"github.com/containerd/containerd/cio"
 	"github.com/containerd/containerd/containers"
 	"github.com/containerd/containerd/errdefs"
-	"github.com/containerd/containerd/images"
-	"github.com/containerd/containerd/log/logtest"
-	"github.com/containerd/containerd/namespaces"
 	"github.com/containerd/containerd/oci"
 	"github.com/containerd/containerd/plugin"
 	"github.com/containerd/containerd/runtime/linux/runctypes"
 	"github.com/containerd/containerd/runtime/v2/runc/options"
 	"github.com/containerd/containerd/sys"
-	"github.com/containerd/typeurl"
-	specs "github.com/opencontainers/runtime-spec/specs-go"
-	"github.com/pkg/errors"
+	"github.com/opencontainers/runtime-spec/specs-go"
+	exec "golang.org/x/sys/execabs"
 	"golang.org/x/sys/unix"
 )
 
 const testUserNSImage = "ghcr.io/containerd/alpine:3.14.0"
 
-// TestRegressionIssue4769 verifies the number of task exit events.
-//
-// Issue: https://github.com/containerd/containerd/issues/4769.
-func TestRegressionIssue4769(t *testing.T) {
-	t.Parallel()
-
-	client, err := newClient(t, address)
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer client.Close()
-
-	// use unique namespace to get unique task events
-	id := t.Name()
-	ns := fmt.Sprintf("%s-%s", testNamespace, id)
-
-	ctx, cancel := context.WithCancel(context.Background())
-	defer cancel()
-
-	ctx = namespaces.WithNamespace(ctx, ns)
-	ctx = logtest.WithT(ctx, t)
-
-	image, err := client.Pull(ctx, testImage, WithPullUnpack)
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer client.ImageService().Delete(ctx, testImage, images.SynchronousDelete())
-
-	container, err := client.NewContainer(ctx, id,
-		WithNewSnapshot(id, image),
-		WithNewSpec(oci.WithImageConfig(image), withTrue()),
-		WithRuntime(client.Runtime(), nil),
-	)
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer container.Delete(ctx, WithSnapshotCleanup)
-
-	task, err := container.NewTask(ctx, empty())
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer task.Delete(ctx)
-
-	statusC, err := task.Wait(ctx)
-	if err != nil {
-		t.Fatal(err)
-	}
-
-	eventStream, errC := client.EventService().Subscribe(ctx, "namespace=="+ns+",topic~=|^/tasks/exit|")
-
-	if err := task.Start(ctx); err != nil {
-		t.Fatal(err)
-	}
-
-	var timeout = 3 * time.Second
-
-	select {
-	case et := <-statusC:
-		if got := et.ExitCode(); got != 0 {
-			t.Fatal(errors.Errorf("expect zero exit status, but got %v", got))
-		}
-	case <-time.After(timeout):
-		t.Fatal(fmt.Errorf("failed to get exit event in time"))
-	}
-
-	// start to check events
-	select {
-	case et := <-eventStream:
-		if et.Event == nil {
-			t.Fatal(errors.Errorf("unexpected empty event: %+v", et))
-		}
-
-		v, err := typeurl.UnmarshalAny(et.Event)
-		if err != nil {
-			t.Fatal(errors.Wrap(err, "failed to unmarshal event"))
-		}
-
-		if e, ok := v.(*apievents.TaskExit); !ok {
-			t.Fatal(errors.Errorf("unexpected event type: %+v", v))
-		} else if e.ExitStatus != 0 {
-			t.Fatal(errors.Errorf("expect zero exit status, but got %v", e.ExitStatus))
-		}
-	case err := <-errC:
-		t.Fatal(errors.Wrap(err, "unexpected error from event service"))
-
-	case <-time.After(timeout):
-		t.Fatal(fmt.Errorf("failed to get exit event in time"))
-	}
-
-	if _, err := task.Delete(ctx); err != nil {
-		t.Fatal(err)
-	}
-
-	// check duplicate event should not show up
-	select {
-	case event := <-eventStream:
-		t.Fatal(errors.Errorf("unexpected exit event: %+v", event))
-	case err := <-errC:
-		t.Fatal(errors.Wrap(err, "unexpected error from event service"))
-	case <-time.After(timeout):
-	}
-}
-
 func TestTaskUpdate(t *testing.T) {
 	t.Parallel()
 
@@ -352,76 +238,6 @@ func TestShimInCgroup(t *testing.T) {
 	<-statusC
 }
 
-func TestDaemonRestart(t *testing.T) {
-	client, err := newClient(t, address)
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer client.Close()
-
-	var (
-		image       Image
-		ctx, cancel = testContext(t)
-		id          = t.Name()
-	)
-	defer cancel()
-
-	image, err = client.GetImage(ctx, testImage)
-	if err != nil {
-		t.Fatal(err)
-	}
-
-	container, err := client.NewContainer(ctx, id, WithNewSnapshot(id, image), WithNewSpec(oci.WithImageConfig(image), withProcessArgs("sleep", "30")))
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer container.Delete(ctx, WithSnapshotCleanup)
-
-	task, err := container.NewTask(ctx, empty())
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer task.Delete(ctx)
-
-	statusC, err := task.Wait(ctx)
-	if err != nil {
-		t.Fatal(err)
-	}
-
-	if err := task.Start(ctx); err != nil {
-		t.Fatal(err)
-	}
-
-	var exitStatus ExitStatus
-	if err := ctrd.Restart(func() {
-		exitStatus = <-statusC
-	}); err != nil {
-		t.Fatal(err)
-	}
-
-	if exitStatus.Error() == nil {
-		t.Errorf(`first task.Wait() should have failed with "transport is closing"`)
-	}
-
-	waitCtx, waitCancel := context.WithTimeout(ctx, 2*time.Second)
-	serving, err := client.IsServing(waitCtx)
-	waitCancel()
-	if !serving {
-		t.Fatalf("containerd did not start within 2s: %v", err)
-	}
-
-	statusC, err = task.Wait(ctx)
-	if err != nil {
-		t.Fatal(err)
-	}
-
-	if err := task.Kill(ctx, syscall.SIGKILL); err != nil {
-		t.Fatal(err)
-	}
-
-	<-statusC
-}
-
 func TestShimDoesNotLeakPipes(t *testing.T) {
 	containerdPid := ctrd.cmd.Process.Pid
 	initialPipes, err := numPipes(containerdPid)
@@ -552,7 +368,7 @@ func TestDaemonReconnectsToShimIOPipesOnRestart(t *testing.T) {
 		t.Fatalf("containerd did not start within 2s: %v", err)
 	}
 
-	// After we restared containerd we write some messages to the log pipes, simulating shim writing stuff there.
+	// After we restarted containerd we write some messages to the log pipes, simulating shim writing stuff there.
 	// Then we make sure that these messages are available on the containerd log thus proving that the server reconnected to the log pipes
 	runtimeVersion := getRuntimeVersion()
 	logDirPath := getLogDirPath(runtimeVersion, id)
@@ -576,7 +392,7 @@ func TestDaemonReconnectsToShimIOPipesOnRestart(t *testing.T) {
 
 	<-statusC
 
-	stdioContents, err := ioutil.ReadFile(ctrdStdioFilePath)
+	stdioContents, err := os.ReadFile(ctrdStdioFilePath)
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -629,76 +445,6 @@ func getRuntimeVersion() string {
 	}
 }
 
-func TestContainerPTY(t *testing.T) {
-	t.Parallel()
-
-	client, err := newClient(t, address)
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer client.Close()
-
-	var (
-		image       Image
-		ctx, cancel = testContext(t)
-		id          = t.Name()
-	)
-	defer cancel()
-
-	image, err = client.GetImage(ctx, testImage)
-	if err != nil {
-		t.Fatal(err)
-	}
-
-	container, err := client.NewContainer(ctx, id, WithNewSnapshot(id, image), WithNewSpec(oci.WithImageConfig(image), oci.WithTTY, withProcessArgs("echo", "hello")))
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer container.Delete(ctx, WithSnapshotCleanup)
-
-	direct, err := newDirectIO(ctx, true)
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer direct.Delete()
-	var (
-		wg  sync.WaitGroup
-		buf = bytes.NewBuffer(nil)
-	)
-	wg.Add(1)
-	go func() {
-		defer wg.Done()
-		io.Copy(buf, direct.Stdout)
-	}()
-
-	task, err := container.NewTask(ctx, direct.IOCreate)
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer task.Delete(ctx)
-
-	status, err := task.Wait(ctx)
-	if err != nil {
-		t.Error(err)
-	}
-
-	if err := task.Start(ctx); err != nil {
-		t.Fatal(err)
-	}
-
-	<-status
-	wg.Wait()
-
-	if err := direct.Close(); err != nil {
-		t.Error(err)
-	}
-
-	out := buf.String()
-	if !strings.ContainsAny(fmt.Sprintf("%#q", out), `\x00`) {
-		t.Fatal(`expected \x00 in output`)
-	}
-}
-
 func TestContainerAttach(t *testing.T) {
 	t.Parallel()
 
@@ -804,126 +550,6 @@ func TestContainerAttach(t *testing.T) {
 	}
 }
 
-func newDirectIO(ctx context.Context, terminal bool) (*directIO, error) {
-	fifos, err := cio.NewFIFOSetInDir("", "", terminal)
-	if err != nil {
-		return nil, err
-	}
-	dio, err := cio.NewDirectIO(ctx, fifos)
-	if err != nil {
-		return nil, err
-	}
-	return &directIO{DirectIO: *dio}, nil
-}
-
-type directIO struct {
-	cio.DirectIO
-}
-
-// ioCreate returns IO available for use with task creation
-func (f *directIO) IOCreate(id string) (cio.IO, error) {
-	return f, nil
-}
-
-// ioAttach returns IO available for use with task attachment
-func (f *directIO) IOAttach(set *cio.FIFOSet) (cio.IO, error) {
-	return f, nil
-}
-
-func (f *directIO) Cancel() {
-	// nothing to cancel as all operations are handled externally
-}
-
-// Close closes all open fds
-func (f *directIO) Close() error {
-	err := f.Stdin.Close()
-	if f.Stdout != nil {
-		if err2 := f.Stdout.Close(); err == nil {
-			err = err2
-		}
-	}
-	if f.Stderr != nil {
-		if err2 := f.Stderr.Close(); err == nil {
-			err = err2
-		}
-	}
-	return err
-}
-
-// Delete removes the underlying directory containing fifos
-func (f *directIO) Delete() error {
-	return f.DirectIO.Close()
-}
-
-func TestContainerUsername(t *testing.T) {
-	t.Parallel()
-
-	client, err := newClient(t, address)
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer client.Close()
-
-	var (
-		image       Image
-		ctx, cancel = testContext(t)
-		id          = t.Name()
-	)
-	defer cancel()
-
-	image, err = client.GetImage(ctx, testImage)
-	if err != nil {
-		t.Fatal(err)
-	}
-	direct, err := newDirectIO(ctx, false)
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer direct.Delete()
-	var (
-		wg  sync.WaitGroup
-		buf = bytes.NewBuffer(nil)
-	)
-	wg.Add(1)
-	go func() {
-		defer wg.Done()
-		io.Copy(buf, direct.Stdout)
-	}()
-
-	// the www-data user in the busybox image has a uid of 33
-	container, err := client.NewContainer(ctx, id,
-		WithNewSnapshot(id, image),
-		WithNewSpec(oci.WithImageConfig(image), oci.WithUsername("www-data"), oci.WithProcessArgs("id", "-u")),
-	)
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer container.Delete(ctx, WithSnapshotCleanup)
-
-	task, err := container.NewTask(ctx, direct.IOCreate)
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer task.Delete(ctx)
-
-	statusC, err := task.Wait(ctx)
-	if err != nil {
-		t.Fatal(err)
-	}
-
-	if err := task.Start(ctx); err != nil {
-		t.Fatal(err)
-	}
-	<-statusC
-
-	wg.Wait()
-
-	output := strings.TrimSuffix(buf.String(), "\n")
-	if output != "33" {
-		t.Errorf("expected www-data uid to be 33 but received %q", output)
-	}
-}
-
 func TestContainerUser(t *testing.T) {
 	t.Parallel()
 	t.Run("UserNameAndGroupName", func(t *testing.T) { testContainerUser(t, "www-data:www-data", "33:33") })
@@ -1486,85 +1112,10 @@ func TestContainerRuntimeOptionsv2(t *testing.T) {
 	}
 }
 
-func initContainerAndCheckChildrenDieOnKill(t *testing.T, opts ...oci.SpecOpts) {
-	client, err := newClient(t, address)
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer client.Close()
-
-	var (
-		image       Image
-		ctx, cancel = testContext(t)
-		id          = t.Name()
-	)
-	defer cancel()
-
-	image, err = client.GetImage(ctx, testImage)
-	if err != nil {
-		t.Fatal(err)
-	}
-
-	opts = append(opts, oci.WithImageConfig(image))
-	opts = append(opts, withProcessArgs("sh", "-c", "sleep 42; echo hi"))
-
-	container, err := client.NewContainer(ctx, id,
-		WithNewSnapshot(id, image),
-		WithNewSpec(opts...),
-	)
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer container.Delete(ctx, WithSnapshotCleanup)
-
-	stdout := bytes.NewBuffer(nil)
-	task, err := container.NewTask(ctx, cio.NewCreator(withByteBuffers(stdout)))
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer task.Delete(ctx)
-
-	statusC, err := task.Wait(ctx)
-	if err != nil {
-		t.Fatal(err)
-	}
-
-	if err := task.Start(ctx); err != nil {
-		t.Fatal(err)
-	}
-
-	if err := task.Kill(ctx, syscall.SIGKILL); err != nil {
-		t.Error(err)
-	}
-
-	// Give the shim time to reap the init process and kill the orphans
-	select {
-	case <-statusC:
-	case <-time.After(100 * time.Millisecond):
-	}
-
-	b, err := exec.Command("ps", "ax").CombinedOutput()
-	if err != nil {
-		t.Fatal(err)
-	}
-
-	if strings.Contains(string(b), "sleep 42") {
-		t.Fatalf("killing init didn't kill all its children:\n%v", string(b))
-	}
-
-	if _, err := task.Delete(ctx, WithProcessKill); err != nil {
-		t.Error(err)
-	}
-}
-
 func TestContainerKillInitPidHost(t *testing.T) {
 	initContainerAndCheckChildrenDieOnKill(t, oci.WithHostNamespace(specs.PIDNamespace))
 }
 
-func TestContainerKillInitKillsChildWhenNotHostPid(t *testing.T) {
-	initContainerAndCheckChildrenDieOnKill(t)
-}
-
 func TestUserNamespaces(t *testing.T) {
 	t.Parallel()
 	t.Run("WritableRootFS", func(t *testing.T) { testUserNamespaces(t, false) })
@@ -1684,110 +1235,6 @@ func testUserNamespaces(t *testing.T, readonlyRootFS bool) {
 	}
 }
 
-func TestTaskResize(t *testing.T) {
-	t.Parallel()
-
-	client, err := newClient(t, address)
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer client.Close()
-
-	var (
-		image       Image
-		ctx, cancel = testContext(t)
-		id          = t.Name()
-	)
-	defer cancel()
-
-	image, err = client.GetImage(ctx, testImage)
-	if err != nil {
-		t.Fatal(err)
-	}
-	container, err := client.NewContainer(ctx, id, WithNewSnapshot(id, image), WithNewSpec(oci.WithImageConfig(image), withExitStatus(7)))
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer container.Delete(ctx, WithSnapshotCleanup)
-
-	task, err := container.NewTask(ctx, empty())
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer task.Delete(ctx)
-
-	statusC, err := task.Wait(ctx)
-	if err != nil {
-		t.Fatal(err)
-	}
-	if err := task.Resize(ctx, 32, 32); err != nil {
-		t.Fatal(err)
-	}
-	task.Kill(ctx, syscall.SIGKILL)
-	<-statusC
-}
-
-func TestContainerImage(t *testing.T) {
-	t.Parallel()
-
-	ctx, cancel := testContext(t)
-	defer cancel()
-	id := t.Name()
-
-	client, err := newClient(t, address)
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer client.Close()
-
-	image, err := client.GetImage(ctx, testImage)
-	if err != nil {
-		t.Fatal(err)
-	}
-
-	container, err := client.NewContainer(ctx, id, WithNewSpec(), WithImage(image))
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer container.Delete(ctx)
-
-	i, err := container.Image(ctx)
-	if err != nil {
-		t.Fatal(err)
-	}
-	if i.Name() != image.Name() {
-		t.Fatalf("expected container image name %s but received %s", image.Name(), i.Name())
-	}
-}
-
-func TestContainerNoImage(t *testing.T) {
-	t.Parallel()
-
-	ctx, cancel := testContext(t)
-	defer cancel()
-	id := t.Name()
-
-	client, err := newClient(t, address)
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer client.Close()
-
-	container, err := client.NewContainer(ctx, id, WithNewSpec())
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer container.Delete(ctx)
-
-	_, err = container.Image(ctx)
-	if err == nil {
-		t.Fatal("error should not be nil when container is created without an image")
-	}
-	if !errdefs.IsNotFound(err) {
-		t.Fatalf("expected error to be %s but received %s", errdefs.ErrNotFound, err)
-	}
-}
-
 func TestUIDNoGID(t *testing.T) {
 	t.Parallel()
 
@@ -1940,55 +1387,6 @@ func TestBindLowPortNonOpt(t *testing.T) {
 	}
 }
 
-func TestContainerNoSTDIN(t *testing.T) {
-	t.Parallel()
-
-	client, err := newClient(t, address)
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer client.Close()
-
-	var (
-		image       Image
-		ctx, cancel = testContext(t)
-		id          = t.Name()
-	)
-	defer cancel()
-
-	image, err = client.GetImage(ctx, testImage)
-	if err != nil {
-		t.Fatal(err)
-	}
-	container, err := client.NewContainer(ctx, id, WithNewSnapshot(id, image), WithNewSpec(oci.WithImageConfig(image), withExitStatus(0)))
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer container.Delete(ctx, WithSnapshotCleanup)
-
-	task, err := container.NewTask(ctx, cio.NewCreator(cio.WithStreams(nil, ioutil.Discard, ioutil.Discard)))
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer task.Delete(ctx)
-
-	statusC, err := task.Wait(ctx)
-	if err != nil {
-		t.Fatal(err)
-	}
-	if err := task.Start(ctx); err != nil {
-		t.Fatal(err)
-	}
-	status := <-statusC
-	code, _, err := status.Result()
-	if err != nil {
-		t.Fatal(err)
-	}
-	if code != 0 {
-		t.Errorf("expected status 0 from wait but received %d", code)
-	}
-}
-
 func TestShimOOMScore(t *testing.T) {
 	containerdPid := ctrd.cmd.Process.Pid
 	containerdScore, err := sys.GetOOMScoreAdj(containerdPid)
@@ -2092,73 +1490,3 @@ func TestShimOOMScore(t *testing.T) {
 
 	<-statusC
 }
-
-func TestTaskSpec(t *testing.T) {
-	t.Parallel()
-
-	client, err := newClient(t, address)
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer client.Close()
-
-	var (
-		image       Image
-		ctx, cancel = testContext(t)
-		id          = t.Name()
-	)
-	defer cancel()
-
-	image, err = client.GetImage(ctx, testImage)
-	if err != nil {
-		t.Fatal(err)
-	}
-
-	container, err := client.NewContainer(ctx, id, WithNewSnapshot(id, image), WithNewSpec(oci.WithImageConfig(image), longCommand))
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer container.Delete(ctx, WithSnapshotCleanup)
-
-	task, err := container.NewTask(ctx, empty())
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer task.Delete(ctx)
-
-	statusC, err := task.Wait(ctx)
-	if err != nil {
-		t.Fatal(err)
-	}
-
-	spec, err := task.Spec(ctx)
-	if err != nil {
-		t.Fatal(err)
-	}
-	if spec == nil {
-		t.Fatal("spec from task is nil")
-	}
-	direct, err := newDirectIO(ctx, false)
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer direct.Delete()
-
-	lt, err := container.Task(ctx, direct.IOAttach)
-	if err != nil {
-		t.Fatal(err)
-	}
-
-	spec, err = lt.Spec(ctx)
-	if err != nil {
-		t.Fatal(err)
-	}
-	if spec == nil {
-		t.Fatal("spec from loaded task is nil")
-	}
-
-	if err := task.Kill(ctx, syscall.SIGKILL); err != nil {
-		t.Fatal(err)
-	}
-	<-statusC
-}
