@@ -26,21 +26,14 @@ import (
 
 	"github.com/containerd/cgroups"
 	"github.com/containerd/containerd/log"
+	"github.com/containerd/containerd/metrics/cgroups/common"
 	v1 "github.com/containerd/containerd/metrics/types/v1"
 	"github.com/containerd/containerd/namespaces"
 	"github.com/containerd/typeurl"
-	metrics "github.com/docker/go-metrics"
-	"github.com/gogo/protobuf/types"
+	"github.com/docker/go-metrics"
 	"github.com/prometheus/client_golang/prometheus"
 )
 
-// Statable type that returns cgroup metrics
-type Statable interface {
-	ID() string
-	Namespace() string
-	Stats(context.Context) (*types.Any, error)
-}
-
 // Trigger will be called when an event happens and provides the cgroup
 // where the event originated from
 type Trigger func(string, string, cgroups.Cgroup)
@@ -54,7 +47,7 @@ func NewCollector(ns *metrics.Namespace) *Collector {
 	// add machine cpus and memory info
 	c := &Collector{
 		ns:    ns,
-		tasks: make(map[string]Statable),
+		tasks: make(map[string]entry),
 	}
 	c.metrics = append(c.metrics, pidMetrics...)
 	c.metrics = append(c.metrics, cpuMetrics...)
@@ -70,15 +63,44 @@ func taskID(id, namespace string) string {
 	return fmt.Sprintf("%s-%s", id, namespace)
 }
 
+type entry struct {
+	task common.Statable
+	// ns is an optional child namespace that contains additional to parent labels.
+	// This can be used to append task specific labels to be able to differentiate the different containerd metrics.
+	ns *metrics.Namespace
+}
+
 // Collector provides the ability to collect container stats and export
 // them in the prometheus format
 type Collector struct {
-	mu sync.RWMutex
-
-	tasks         map[string]Statable
 	ns            *metrics.Namespace
-	metrics       []*metric
 	storedMetrics chan prometheus.Metric
+
+	// TODO(fuweid):
+	//
+	// The Collector.Collect will be the field ns'Collect's callback,
+	// which be invoked periodically with internal lock. And Collector.Add
+	// might also invoke ns.Lock if the labels is not nil, which is easy to
+	// cause dead-lock.
+	//
+	// Goroutine X:
+	//
+	//	ns.Collect
+	//   	  ns.Lock
+	//          Collector.Collect
+	//            Collector.RLock
+	//
+	//
+	// Goroutine Y:
+	//
+	//	Collector.Add
+	//        ...(RLock/Lock)
+	//	    ns.Lock
+	//
+	// I think we should seek the way to decouple ns from Collector.
+	mu      sync.RWMutex
+	tasks   map[string]entry
+	metrics []*metric
 }
 
 // Describe prometheus metrics
@@ -110,10 +132,11 @@ storedLoop:
 	wg.Wait()
 }
 
-func (c *Collector) collect(t Statable, ch chan<- prometheus.Metric, block bool, wg *sync.WaitGroup) {
+func (c *Collector) collect(entry entry, ch chan<- prometheus.Metric, block bool, wg *sync.WaitGroup) {
 	if wg != nil {
 		defer wg.Done()
 	}
+	t := entry.task
 	ctx := namespaces.WithNamespace(context.Background(), t.Namespace())
 	stats, err := t.Stats(ctx)
 	if err != nil {
@@ -130,28 +153,41 @@ func (c *Collector) collect(t Statable, ch chan<- prometheus.Metric, block bool,
 		log.L.WithError(err).Errorf("invalid metric type for %s", t.ID())
 		return
 	}
+	ns := entry.ns
+	if ns == nil {
+		ns = c.ns
+	}
 	for _, m := range c.metrics {
-		m.collect(t.ID(), t.Namespace(), s, c.ns, ch, block)
+		m.collect(t.ID(), t.Namespace(), s, ns, ch, block)
 	}
 }
 
 // Add adds the provided cgroup and id so that metrics are collected and exported
-func (c *Collector) Add(t Statable) error {
+func (c *Collector) Add(t common.Statable, labels map[string]string) error {
 	if c.ns == nil {
 		return nil
 	}
-	c.mu.Lock()
-	defer c.mu.Unlock()
+	c.mu.RLock()
 	id := taskID(t.ID(), t.Namespace())
-	if _, ok := c.tasks[id]; ok {
+	_, ok := c.tasks[id]
+	c.mu.RUnlock()
+	if ok {
 		return nil // requests to collect metrics should be idempotent
 	}
-	c.tasks[id] = t
+
+	entry := entry{task: t}
+	if labels != nil {
+		entry.ns = c.ns.WithConstLabels(labels)
+	}
+
+	c.mu.Lock()
+	c.tasks[id] = entry
+	c.mu.Unlock()
 	return nil
 }
 
 // Remove removes the provided cgroup by id from the collector
-func (c *Collector) Remove(t Statable) {
+func (c *Collector) Remove(t common.Statable) {
 	if c.ns == nil {
 		return
 	}
@@ -166,6 +202,6 @@ func (c *Collector) RemoveAll() {
 		return
 	}
 	c.mu.Lock()
-	c.tasks = make(map[string]Statable)
+	c.tasks = make(map[string]entry)
 	c.mu.Unlock()
 }
