@@ -18,7 +18,6 @@ package oci
 
 import (
 	"context"
-	"io/ioutil"
 	"os"
 	"path/filepath"
 	"testing"
@@ -40,7 +39,6 @@ func TestAddCaps(t *testing.T) {
 		s.Process.Capabilities.Bounding,
 		s.Process.Capabilities.Effective,
 		s.Process.Capabilities.Permitted,
-		s.Process.Capabilities.Inheritable,
 	} {
 		if !capsContain(cl, "CAP_CHOWN") {
 			t.Errorf("cap list %d does not contain added cap", i)
@@ -64,7 +62,6 @@ func TestDropCaps(t *testing.T) {
 		s.Process.Capabilities.Bounding,
 		s.Process.Capabilities.Effective,
 		s.Process.Capabilities.Permitted,
-		s.Process.Capabilities.Inheritable,
 	} {
 		if capsContain(cl, "CAP_CHOWN") {
 			t.Errorf("cap list %d contains dropped cap", i)
@@ -83,7 +80,6 @@ func TestDropCaps(t *testing.T) {
 		s.Process.Capabilities.Bounding,
 		s.Process.Capabilities.Effective,
 		s.Process.Capabilities.Permitted,
-		s.Process.Capabilities.Inheritable,
 	} {
 		if capsContain(cl, "CAP_FOWNER") {
 			t.Errorf("cap list %d contains dropped cap", i)
@@ -104,7 +100,6 @@ func TestDropCaps(t *testing.T) {
 		s.Process.Capabilities.Bounding,
 		s.Process.Capabilities.Effective,
 		s.Process.Capabilities.Permitted,
-		s.Process.Capabilities.Inheritable,
 	} {
 		if len(cl) != 0 {
 			t.Errorf("cap list %d is not empty", i)
@@ -115,14 +110,14 @@ func TestDropCaps(t *testing.T) {
 func TestGetDevices(t *testing.T) {
 	testutil.RequiresRoot(t)
 
-	dir, err := ioutil.TempDir("/dev", t.Name())
+	dir, err := os.MkdirTemp("/dev", t.Name())
 	if err != nil {
 		t.Fatal(err)
 	}
 	defer os.RemoveAll(dir)
 
 	zero := filepath.Join(dir, "zero")
-	if err := ioutil.WriteFile(zero, nil, 0600); err != nil {
+	if err := os.WriteFile(zero, nil, 0600); err != nil {
 		t.Fatal(err)
 	}
 
@@ -162,7 +157,7 @@ func TestGetDevices(t *testing.T) {
 	})
 	t.Run("two devices", func(t *testing.T) {
 		nullDev := filepath.Join(dir, "null")
-		if err := ioutil.WriteFile(nullDev, nil, 0600); err != nil {
+		if err := os.WriteFile(nullDev, nil, 0600); err != nil {
 			t.Fatal(err)
 		}
 
@@ -237,7 +232,7 @@ func TestGetDevices(t *testing.T) {
 			}
 		})
 		t.Run("regular file in dir", func(t *testing.T) {
-			if err := ioutil.WriteFile(filepath.Join(dir, "somefile"), []byte("hello"), 0600); err != nil {
+			if err := os.WriteFile(filepath.Join(dir, "somefile"), []byte("hello"), 0600); err != nil {
 				t.Fatal(err)
 			}
 			defer os.Remove(filepath.Join(dir, "somefile"))
