@@ -18,8 +18,12 @@ package opts
 
 import (
 	"context"
-	"io/ioutil"
+	"errors"
+	"fmt"
 	"os"
+	"path/filepath"
+	goruntime "runtime"
+	"strings"
 
 	"github.com/containerd/containerd"
 	"github.com/containerd/containerd/containers"
@@ -28,7 +32,6 @@ import (
 	"github.com/containerd/containerd/mount"
 	"github.com/containerd/containerd/snapshots"
 	"github.com/containerd/continuity/fs"
-	"github.com/pkg/errors"
 )
 
 // WithNewSnapshot wraps `containerd.WithNewSnapshot` so that if creating the
@@ -42,7 +45,7 @@ func WithNewSnapshot(id string, i containerd.Image, opts ...snapshots.Opt) conta
 			}
 
 			if err := i.Unpack(ctx, c.Snapshotter); err != nil {
-				return errors.Wrap(err, "error unpacking image")
+				return fmt.Errorf("error unpacking image: %w", err)
 			}
 			return f(ctx, client, c)
 		}
@@ -66,7 +69,13 @@ func WithVolumes(volumeMounts map[string]string) containerd.NewContainerOpts {
 		if err != nil {
 			return err
 		}
-		root, err := ioutil.TempDir("", "ctd-volume")
+		// Since only read is needed, append ReadOnly mount option to prevent linux kernel
+		// from syncing whole filesystem in umount syscall.
+		if len(mounts) == 1 && mounts[0].Type == "overlay" {
+			mounts[0].Options = append(mounts[0].Options, "ro")
+		}
+
+		root, err := os.MkdirTemp("", "ctd-volume")
 		if err != nil {
 			return err
 		}
@@ -74,33 +83,55 @@ func WithVolumes(volumeMounts map[string]string) containerd.NewContainerOpts {
 		// if it fails but not RM snapshot data.
 		// refer to https://github.com/containerd/containerd/pull/1868
 		// https://github.com/containerd/containerd/pull/1785
-		defer os.Remove(root) // nolint: errcheck
-		if err := mount.All(mounts, root); err != nil {
-			return errors.Wrap(err, "failed to mount")
-		}
-		defer func() {
-			if uerr := mount.Unmount(root, 0); uerr != nil {
-				log.G(ctx).WithError(uerr).Errorf("Failed to unmount snapshot %q", c.SnapshotKey)
+		defer os.Remove(root) //nolint:errcheck
+
+		unmounter := func(mountPath string) {
+			if uerr := mount.Unmount(mountPath, 0); uerr != nil {
+				log.G(ctx).WithError(uerr).Errorf("Failed to unmount snapshot %q", root)
 				if err == nil {
 					err = uerr
 				}
 			}
-		}()
+		}
+
+		var mountPaths []string
+		if goruntime.GOOS == "windows" {
+			for _, m := range mounts {
+				// appending the layerID to the root.
+				mountPath := filepath.Join(root, filepath.Base(m.Source))
+				mountPaths = append(mountPaths, mountPath)
+				if err := m.Mount(mountPath); err != nil {
+					return err
+				}
+
+				defer unmounter(m.Source)
+			}
+		} else {
+			mountPaths = append(mountPaths, root)
+			if err := mount.All(mounts, root); err != nil {
+				return fmt.Errorf("failed to mount: %w", err)
+			}
+			defer unmounter(root)
+		}
 
 		for host, volume := range volumeMounts {
-			src, err := fs.RootPath(root, volume)
-			if err != nil {
-				return errors.Wrapf(err, "rootpath on root %s, volume %s", root, volume)
-			}
-			if _, err := os.Stat(src); err != nil {
-				if os.IsNotExist(err) {
-					// Skip copying directory if it does not exist.
-					continue
+			// The volume may have been defined with a C: prefix, which we can't use here.
+			volume = strings.TrimPrefix(volume, "C:")
+			for _, mountPath := range mountPaths {
+				src, err := fs.RootPath(mountPath, volume)
+				if err != nil {
+					return fmt.Errorf("rootpath on mountPath %s, volume %s: %w", mountPath, volume, err)
+				}
+				if _, err := os.Stat(src); err != nil {
+					if os.IsNotExist(err) {
+						// Skip copying directory if it does not exist.
+						continue
+					}
+					return fmt.Errorf("stat volume in rootfs: %w", err)
+				}
+				if err := copyExistingContents(src, host); err != nil {
+					return fmt.Errorf("taking runtime copy of volume: %w", err)
 				}
-				return errors.Wrap(err, "stat volume in rootfs")
-			}
-			if err := copyExistingContents(src, host); err != nil {
-				return errors.Wrap(err, "taking runtime copy of volume")
 			}
 		}
 		return nil
@@ -110,12 +141,12 @@ func WithVolumes(volumeMounts map[string]string) containerd.NewContainerOpts {
 // copyExistingContents copies from the source to the destination and
 // ensures the ownership is appropriately set.
 func copyExistingContents(source, destination string) error {
-	dstList, err := ioutil.ReadDir(destination)
+	dstList, err := os.ReadDir(destination)
 	if err != nil {
 		return err
 	}
 	if len(dstList) != 0 {
-		return errors.Errorf("volume at %q is not initially empty", destination)
+		return fmt.Errorf("volume at %q is not initially empty", destination)
 	}
 	return fs.CopyDir(destination, source, fs.WithXAttrExclude("security.selinux"))
 }
