@@ -1,3 +1,4 @@
+//go:build !linux && !windows
 // +build !linux,!windows
 
 /*
@@ -19,8 +20,9 @@
 package client
 
 import (
-	"os/exec"
 	"syscall"
+
+	exec "golang.org/x/sys/execabs"
 )
 
 func getSysProcAttr() *syscall.SysProcAttr {
