@@ -14,34 +14,42 @@
 #   See the License for the specific language governing permissions and
 #   limitations under the License.
 
+set -o errexit
 set -o nounset
 set -o pipefail
 
-source $(dirname "${BASH_SOURCE[0]}")/utils.sh
-cd ${ROOT}
+basedir="$(dirname "${BASH_SOURCE[0]}")"
+source "${basedir}/utils.sh"
+
+trap test_teardown EXIT
+
+ROOT="$( cd "${basedir}" && pwd )"/../..
+cd "${ROOT}"
 
 # FOCUS focuses the test to run.
 FOCUS=${FOCUS:-""}
 # REPORT_DIR is the the directory to store test logs.
-REPORT_DIR=${REPORT_DIR:-"/tmp/test-integration"}
+if [ $IS_WINDOWS -eq 0 ]; then
+  REPORT_DIR=${REPORT_DIR:-"/tmp/test-integration"}
+else
+  REPORT_DIR=${REPORT_DIR:-"C:/Windows/Temp/test-integration"}
+fi
 # RUNTIME is the runtime handler to use in the test.
 RUNTIME=${RUNTIME:-""}
 
 CRI_ROOT="${CONTAINERD_ROOT}/io.containerd.grpc.v1.cri"
-
-mkdir -p ${REPORT_DIR}
-test_setup ${REPORT_DIR}
+mkdir -p "${REPORT_DIR}"
+test_setup "${REPORT_DIR}"
 
 # Run integration test.
-sudo PATH=${PATH} bin/cri-integration.test --test.run="${FOCUS}" --test.v \
-  --cri-endpoint=${CONTAINERD_SOCK} \
-  --cri-root=${CRI_ROOT} \
-  --runtime-handler=${RUNTIME} \
-  --containerd-bin=${CONTAINERD_BIN} \
-  --image-list="${TEST_IMAGE_LIST:-}"
+${sudo} bin/cri-integration.test --test.run="${FOCUS}" --test.v \
+  --cri-endpoint="${CONTAINERD_SOCK}" \
+  --cri-root="${CRI_ROOT}" \
+  --runtime-handler="${RUNTIME}" \
+  --containerd-bin="${CONTAINERD_BIN}" \
+  --image-list="${TEST_IMAGE_LIST:-}" && test_exit_code=$? || test_exit_code=$?
 
-test_exit_code=$?
-
-test_teardown
+test $test_exit_code -ne 0 && \
+  cat "$REPORT_DIR/containerd.log"
 
 exit ${test_exit_code}
