@@ -17,19 +17,38 @@
 package server
 
 import (
-	"context"
 	"testing"
 
 	srvconfig "github.com/containerd/containerd/services/server/config"
-	"gotest.tools/assert"
-	is "gotest.tools/assert/cmp"
+	"gotest.tools/v3/assert"
+	is "gotest.tools/v3/assert/cmp"
 )
 
-func TestNewErrorsWithSamePathForRootAndState(t *testing.T) {
+func TestCreateTopLevelDirectoriesErrorsWithSamePathForRootAndState(t *testing.T) {
 	path := "/tmp/path/for/testing"
-	_, err := New(context.Background(), &srvconfig.Config{
+	err := CreateTopLevelDirectories(&srvconfig.Config{
 		Root:  path,
 		State: path,
 	})
 	assert.Check(t, is.Error(err, "root and state must be different paths"))
 }
+
+func TestCreateTopLevelDirectoriesWithEmptyStatePath(t *testing.T) {
+	statePath := ""
+	rootPath := "/tmp/path/for/testing"
+	err := CreateTopLevelDirectories(&srvconfig.Config{
+		Root:  rootPath,
+		State: statePath,
+	})
+	assert.Check(t, is.Error(err, "state must be specified"))
+}
+
+func TestCreateTopLevelDirectoriesWithEmptyRootPath(t *testing.T) {
+	statePath := "/tmp/path/for/testing"
+	rootPath := ""
+	err := CreateTopLevelDirectories(&srvconfig.Config{
+		Root:  rootPath,
+		State: statePath,
+	})
+	assert.Check(t, is.Error(err, "root must be specified"))
+}
