@@ -1,6 +1,3 @@
-//go:build freebsd
-// +build freebsd
-
 /*
    Copyright The containerd Authors.
 
@@ -25,6 +22,7 @@ import (
 )
 
 var tasksServiceRequires = []plugin.Type{
+	plugin.EventPlugin,
 	plugin.RuntimePluginV2,
 	plugin.MetadataPlugin,
 	plugin.TaskMonitorPlugin,
